var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { asMaybePasswordError, asMaybeUsernameError } from 'edge-core-js';
import * as React from 'react';
import { Keyboard, ScrollView, TouchableOpacity, View } from 'react-native';
import { KeyboardAwareScrollView } from 'react-native-keyboard-aware-scroll-view';
import { cacheStyles } from 'react-native-patina';
import Animated, { Easing, useAnimatedStyle, useDerivedValue, useSharedValue, withTiming } from 'react-native-reanimated';
import AntDesignIcon from 'react-native-vector-icons/AntDesign';
import MaterialIcon from 'react-native-vector-icons/MaterialIcons';
import { sprintf } from 'sprintf-js';
import { launchPasswordRecovery, login } from '../../actions/LoginAction';
import { maybeRouteComplete } from '../../actions/LoginInitActions';
import s from '../../common/locales/strings';
import { useHandler } from '../../hooks/useHandler';
import { useImports } from '../../hooks/useImports';
import { useLocalUsers } from '../../hooks/useLocalUsers';
import { useDispatch, useSelector } from '../../types/ReduxTypes';
import { logEvent } from '../../util/analytics';
import { base58 } from '../../util/base58';
import { LogoImageHeader } from '../abSpecific/LogoImageHeader';
import { UserListItem } from '../abSpecific/UserListItem';
import { ButtonsModal } from '../modals/ButtonsModal';
import { showQrCodeModal } from '../modals/QrCodeModal';
import { TextInputModal } from '../modals/TextInputModal';
import { Airship, showError } from '../services/AirshipInstance';
import { useTheme } from '../services/ThemeContext';
import { MainButton } from '../themed/MainButton';
import { OutlinedTextInput } from '../themed/OutlinedTextInput';
import { ThemedScene } from '../themed/ThemedScene';
// Non-round number so that the scroll is apparent with more than max displayed
// localUsers
const MAX_DISPLAYED_LOCAL_USERS = 4.75;
export const PasswordLoginScene = (props) => {
    const { branding, route } = props;
    const { username } = route.params;
    const { context } = useImports();
    const dispatch = useDispatch();
    const theme = useTheme();
    const styles = getStyles(theme);
    const localUsers = useLocalUsers();
    const touch = useSelector(state => state.touch.type);
    const isMultiLocalUsers = localUsers.length > 1;
    const [passwordErrorMessage, setPasswordErrorMessage] = React.useState(undefined);
    const [usernameErrorMessage, setUsernameErrorMessage] = React.useState(undefined);
    const [password, setPassword] = React.useState('');
    const [showUsernameList, setShowUsernameList] = React.useState(false);
    const [dropdownY, setDropdownY] = React.useState(0);
    const passwordInputRef = React.useRef(null);
    const [usernameItemHeight, setUsernameItemHeight] = React.useState(0);
    const mDropContainerStyle = React.useMemo(() => {
        return Object.assign({ top: dropdownY }, styles.dropContainer);
    }, [styles, dropdownY]);
    const sAnimationMult = useSharedValue(0);
    const dFinalHeight = useDerivedValue(() => {
        return (usernameItemHeight *
            Math.min(localUsers.length, MAX_DISPLAYED_LOCAL_USERS));
    }, [usernameItemHeight, localUsers]);
    const aDropContainerStyle = useAnimatedStyle(() => ({
        height: dFinalHeight.value * sAnimationMult.value,
        opacity: showUsernameList
            ? withTiming(1, { duration: 50, easing: Easing.exp })
            : withTiming(0, { duration: 200, easing: Easing.exp })
    }), [showUsernameList]);
    const handleUsernameLayout = useHandler((event) => {
        setDropdownY(event.nativeEvent.layout.y + theme.rem(3.5));
    });
    const handleDropdownItemLayout = useHandler((event) => {
        if (event != null && usernameItemHeight === 0) {
            const { height } = event.nativeEvent.layout;
            setUsernameItemHeight(height);
        }
    });
    const handlePasswordChange = useHandler((password) => {
        setPasswordErrorMessage(undefined);
        setPassword(password);
    });
    const handleSubmit = useHandler(() => __awaiter(void 0, void 0, void 0, function* () {
        Keyboard.dismiss();
        const otpAttempt = { type: 'password', username, password };
        yield dispatch(login(otpAttempt)).catch(error => {
            if (error != null && error.name === 'OtpError') {
                dispatch({
                    type: 'NAVIGATE',
                    data: { name: 'otpError', params: { otpAttempt, otpError: error } }
                });
            }
            else {
                console.log(error);
                const usernameError = asMaybeUsernameError(error);
                if (usernameError != null) {
                    setUsernameErrorMessage(s.strings.invalid_account);
                    return;
                }
                const passwordError = asMaybePasswordError(error);
                if (passwordError != null) {
                    setPasswordErrorMessage(s.strings.invalid_password);
                    return;
                }
                console.warn('Unknown login error: ', error);
                const unknownErrorMessage = error != null ? error.message : undefined;
                setPasswordErrorMessage(unknownErrorMessage);
            }
        });
    }));
    const handleBack = useHandler(() => {
        dispatch(maybeRouteComplete({
            type: 'NAVIGATE',
            data: { name: 'landing', params: {} }
        }));
    });
    const handleDelete = useHandler((userInfo) => {
        Keyboard.dismiss();
        Airship.show(bridge => {
            var _a;
            return (React.createElement(ButtonsModal, { bridge: bridge, title: s.strings.forget_account, message: sprintf(s.strings.forget_username_account, (_a = userInfo.username) !== null && _a !== void 0 ? _a : s.strings.username), buttons: {
                    ok: { label: s.strings.forget },
                    cancel: { label: s.strings.cancel, type: 'secondary' }
                } }));
        })
            .then((button) => __awaiter(void 0, void 0, void 0, function* () {
            if (button !== 'ok')
                return;
            if (context.forgetAccount != null) {
                yield context.forgetAccount(userInfo.loginId);
            }
            else {
                const { username } = userInfo;
                if (username == null)
                    throw new Error('No username');
                yield context.deleteLocalAccount(username);
            }
        }))
            .catch(showError);
    });
    const handleToggleUsernameList = useHandler(() => {
        setShowUsernameList(!showUsernameList);
    });
    const handleSelectUser = useHandler((userInfo) => {
        const { loginId, username } = userInfo;
        if (username == null)
            return; // These don't exist in the list
        handleChangeUsername(username);
        setShowUsernameList(false);
        const details = localUsers.find(info => info.username === username);
        if (details != null &&
            (details.pinLoginEnabled || (details.touchLoginEnabled && touch))) {
            dispatch({
                type: 'NAVIGATE',
                data: { name: 'pinLogin', params: { loginId } }
            });
        }
        else {
            if (passwordInputRef.current != null)
                passwordInputRef.current.focus();
        }
    });
    const handleChangeUsername = useHandler((username) => {
        setPasswordErrorMessage(undefined);
        setUsernameErrorMessage(undefined);
        dispatch({
            type: 'NAVIGATE',
            data: { name: 'passwordLogin', params: { username } }
        });
    });
    const handleSubmitRecoveryKey = useHandler((recoveryKey) => __awaiter(void 0, void 0, void 0, function* () {
        var _a;
        if (((_a = base58.parseUnsafe(recoveryKey)) === null || _a === void 0 ? void 0 : _a.length) !== 32)
            return s.strings.recovery_token_invalid;
        dispatch(launchPasswordRecovery(recoveryKey));
        return true;
    }));
    const handleForgotPassword = useHandler(() => {
        Keyboard.dismiss();
        logEvent('Login_Password_Forgot_Password');
        Airship.show(bridge => (React.createElement(TextInputModal, { bridge: bridge, onSubmit: handleSubmitRecoveryKey, title: s.strings.password_recovery, message: s.strings.initiate_password_recovery, inputLabel: s.strings.recovery_token })));
    });
    const handleCreateAccount = useHandler(() => {
        logEvent('Login_Password_Create_Account');
        dispatch({
            type: 'NAVIGATE',
            data: { name: 'newAccountWelcome', params: {} }
        });
    });
    const handleQrModal = useHandler(() => {
        Keyboard.dismiss();
        dispatch(showQrCodeModal());
    });
    // The main hints dropdown animation depending on focus state of the
    // username dropdown
    React.useEffect(() => {
        sAnimationMult.value = withTiming(showUsernameList ? 1 : 0, {
            duration: 250,
            easing: Easing.inOut(Easing.circle)
        });
    }, [sAnimationMult, showUsernameList]);
    const renderUsername = () => (React.createElement(View, { style: styles.usernameWrapper },
        React.createElement(View, { style: styles.inputField, onLayout: handleUsernameLayout },
            React.createElement(OutlinedTextInput, { autoCorrect: false, autoFocus: true, clearIcon: !isMultiLocalUsers, error: usernameErrorMessage, label: s.strings.username, marginRem: [0.5, 1, 0.5, 1], returnKeyType: "next", testID: "usernameFormField", value: username, onChangeText: handleChangeUsername })),
        isMultiLocalUsers ? (React.createElement(TouchableOpacity, { testID: "userDropdownIcon", style: styles.dropdownButton, onPress: handleToggleUsernameList }, showUsernameList ? (React.createElement(MaterialIcon, { name: "expand-less", size: theme.rem(1.5), style: styles.iconColor })) : (React.createElement(MaterialIcon, { name: "expand-more", size: theme.rem(1.5), style: styles.iconColor })))) : null));
    const renderDropdownList = () => {
        return (React.createElement(Animated.View, { style: [mDropContainerStyle, aDropContainerStyle] },
            React.createElement(ScrollView, { keyboardShouldPersistTaps: "handled" }, localUsers.map(userInfo => {
                const { username } = userInfo;
                if (username == null)
                    return null;
                return (React.createElement(UserListItem, { key: username, userInfo: userInfo, onClick: handleSelectUser, onDelete: handleDelete, onLayout: handleDropdownItemLayout }));
            }))));
    };
    const renderPassword = () => {
        return (React.createElement(View, { style: styles.inputField },
            React.createElement(OutlinedTextInput, { ref: passwordInputRef, autoCorrect: false, autoFocus: false, error: passwordErrorMessage, label: s.strings.password, marginRem: [0.5, 1, 0.5, 1], returnKeyType: "done", secureTextEntry: true, testID: "passwordFormField", value: password, onChangeText: handlePasswordChange, onSubmitEditing: handleSubmit })));
    };
    const renderButtons = () => {
        const buttonType = theme.preferPrimaryButton ? 'primary' : 'secondary';
        return (React.createElement(View, { style: styles.buttonsBox },
            React.createElement(MainButton, { type: "textOnly", onPress: handleForgotPassword, label: s.strings.forgot_password }),
            React.createElement(View, { style: styles.loginButtonBox },
                React.createElement(MainButton, { label: s.strings.login_button, testID: "loginButton", type: buttonType, disabled: username.length === 0 ||
                        password.length === 0 ||
                        usernameErrorMessage != null ||
                        passwordErrorMessage != null, onPress: handleSubmit })),
            React.createElement(MainButton, { type: "textOnly", testID: "createAccountButton", onPress: handleCreateAccount, label: s.strings.create_an_account }),
            React.createElement(TouchableOpacity, { onPress: handleQrModal },
                React.createElement(AntDesignIcon, { name: "qrcode", color: theme.icon, size: theme.rem(1.75) }))));
    };
    return (React.createElement(ThemedScene, { onBack: handleBack, noUnderline: true, branding: branding },
        React.createElement(KeyboardAwareScrollView, { style: styles.container, keyboardShouldPersistTaps: "always" },
            React.createElement(LogoImageHeader, { branding: branding }),
            React.createElement(View, { style: styles.inputContainer },
                renderUsername(),
                renderDropdownList(),
                renderPassword(),
                renderButtons()))));
};
const getStyles = cacheStyles((theme) => ({
    dropContainer: {
        backgroundColor: theme.modal,
        borderRadius: theme.rem(0.5),
        zIndex: 1,
        borderColor: theme.iconTappable,
        borderWidth: theme.thinLineWidth,
        overflow: 'hidden',
        position: 'absolute',
        marginHorizontal: theme.rem(1),
        flex: 1
    },
    container: {
        flex: 1,
        paddingTop: theme.rem(2),
        paddingHorizontal: theme.rem(0.5)
    },
    inputContainer: {
        marginHorizontal: theme.rem(0.5),
        marginTop: theme.rem(2)
    },
    loginButtonBox: {
        marginVertical: theme.rem(0.25),
        width: '70%'
    },
    buttonsBox: {
        alignItems: 'center'
    },
    usernameWrapper: {
        flexDirection: 'row'
    },
    inputField: {
        flex: 1,
        marginBottom: theme.rem(1)
    },
    // TODO: Integrate dropdown into OutlinedTextInput
    dropdownButton: {
        justifyContent: 'center',
        alignItems: 'center',
        position: 'absolute',
        right: theme.rem(1.25),
        width: theme.rem(2.5),
        height: theme.rem(2),
        bottom: theme.rem(1.85)
    },
    iconColor: {
        color: theme.iconTappable
    }
}));
