import { EdgeAccount } from 'edge-core-js';
import { ChangeRecoveryParams } from '../components/scenes/existingAccout/ChangeRecoveryScene';
import { OtpRepairParams } from '../components/scenes/existingAccout/OtpRepairScene';
import { NewAcountReviewParams } from '../components/scenes/newAccount/NewAccountReviewScene';
import { OtpErrorParams } from '../components/scenes/OtpErrorScene';
import { PasswordLoginParams } from '../components/scenes/PasswordLoginScene';
import { PinLoginParams } from '../components/scenes/PinLoginScene';
import { RecoveryLoginParams } from '../components/scenes/RecoveryLoginScene';
import { WaitParams } from '../components/scenes/WaitScene';
export interface AccountParams {
    account: EdgeAccount;
}
/**
 * Data passed between scenes for the create account flows.
 */
export interface CreateFlowParams {
    username?: string;
    password?: string;
    pin?: string;
}
/**
 * Defines the acceptable route parameters for each scene key.
 */
export interface LoginParamList {
    changePassword: AccountParams;
    changePin: AccountParams;
    changeRecovery: ChangeRecoveryParams;
    newAccountWelcome: {};
    newAccountUsername: CreateFlowParams;
    newAccountPassword: CreateFlowParams;
    newAccountPin: CreateFlowParams;
    newAccountTos: CreateFlowParams;
    newAccountWait: WaitParams;
    newAccountReview: NewAcountReviewParams;
    landing: {};
    loading: {};
    otpError: OtpErrorParams;
    otpRepair: OtpRepairParams;
    passwordLogin: PasswordLoginParams;
    pinLogin: PinLoginParams;
    recoveryLogin: RecoveryLoginParams;
    resecurePassword: AccountParams;
    resecurePin: AccountParams;
    securityAlert: AccountParams;
}
/**
 * The `route` prop passed to each scene.
 */
export interface RouteProp<Name extends keyof LoginParamList> {
    name: Name;
    params: LoginParamList[Name];
}
/**
 * The props passed to each scene.
 */
export interface SceneProps<Name extends keyof LoginParamList> {
    route: RouteProp<Name>;
}
